//uniraum.h:

/*
 *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2011
 *
 *  This file is part of uniraum,
 *  a cross-platform container library, home and professional use.
 *  See README for details.
 *
 *  This file is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  uniraum is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#ifndef _UNIRAUM_UNIRAUM_H_
#define _UNIRAUM_UNIRAUM_H_

#include <stdint.h>
#include <roaraudio.h>

typedef struct uniraum_file      uniraum_file_t;
typedef struct uniraum_req       uniraum_req_t;
typedef struct uniraum_stream    uniraum_stream_t;
typedef struct uniraum_mapping   uniraum_mapping_t;
typedef struct uniraum_seektable uniraum_seektable_t;
typedef struct uniraum_seekpoint uniraum_seekpoint_t;
typedef int64_t                  uniraum_seekpos_t;

typedef enum {
 UNIRAUM_SEEK_SET = SEEK_SET,
 UNIRAUM_SEEK_ABS = UNIRAUM_SEEK_SET,
 UNIRAUM_SEEK_CUR = SEEK_CUR,
 UNIRAUM_SEEK_REL = UNIRAUM_SEEK_CUR,
 UNIRAUM_SEEK_END = SEEK_END
} uniraum_whence_t;

typedef enum {
 UNIRAUM_ERROR = -1,
 UNIRAUM_NONE  =  0,
 UNIRAUM_IN    =  1,
 UNIRAUM_OUT   =  2,
 UNIRAUM_BIDIR =  3
} uniraum_dir_t;

typedef uint64_t               uniraum_pos_t;
typedef uint32_t               uniraum_cap_t;
typedef struct roar_buffer     uniraum_data_t;

#define UNIRAUM__DATA_CAPS_MINIMUM_LEN ((size_t)2)
#define UNIRAUM_CAPS_MINIMUM uniraum__data_caps_minimum, UNIRAUM__DATA_CAPS_MINIMUM_LEN
extern uniraum_cap_t uniraum__data_caps_minimum[UNIRAUM__DATA_CAPS_MINIMUM_LEN];

// continer level functions:

uniraum_file_t * uniraum_openvio(struct roar_vio_calls * vio, int flags);
uniraum_file_t * uniraum_opendstr(const char * dstr, struct roar_vio_defaults * def, int flags);
int              uniraum_ref(uniraum_file_t * state);
int              uniraum_unref(uniraum_file_t * state);
#define          uniraum_close(x) uniraum_unref((x))
int              uniraum_destroy(uniraum_file_t * state);

int              uniraum_error(uniraum_file_t * state);

int              uniraum_read(uniraum_file_t * state);
uniraum_req_t *  uniraum_reqin(uniraum_file_t * state);
int              uniraum_reqout(uniraum_file_t * state, uniraum_req_t * req);
int              uniraum_write(uniraum_file_t * state);

int              uniraum_get_execed(uniraum_file_t * state);

ssize_t          uniraum_write_execed(uniraum_file_t * state, const void * buf, size_t len);
ssize_t          uniraum_read_execed (uniraum_file_t * state,       void * buf, size_t len);

// for seeking:
uniraum_seekpos_t uniraum_get_seekpos(uniraum_file_t * state);
int               uniraum_seek(uniraum_file_t * state, uniraum_seekpos_t pos, uniraum_whence_t whence);

// req level functions:
uniraum_req_t *  uniraum_reqexec(int stream, uniraum_pos_t pos);
uniraum_req_t *  uniraum_reqnoop(size_t len, int stream, uniraum_pos_t pos, int random);
uniraum_req_t *  uniraum_reqpadding(size_t len);
uniraum_req_t *  uniraum_reqeof(void);

uniraum_req_t *  uniraum_reqnew_stream(int stream, int parent, int dir, struct roar_audio_info * info);
uniraum_req_t *  uniraum_reqnew_stream_simple(int * stream, uint_least32_t rate, uint_least32_t bits, uint_least32_t channels, uint_least32_t codec);
uniraum_req_t *  uniraum_reqend_stream(int stream, uniraum_pos_t pos);

uniraum_req_t *  uniraum_reqset_meta(int stream, const char * key, const char * value);
uniraum_req_t *  uniraum_reqadd_meta(int stream, const char * key, const char * value);
uniraum_req_t *  uniraum_reqfin_meta(int stream);
uniraum_req_t *  uniraum_reqclear_meta(int stream);

uniraum_req_t *  uniraum_reqdata(int stream, uniraum_pos_t pos, const void * data, size_t len);

uniraum_req_t *  uniraum_reqvolume(int stream, ...);

uniraum_req_t *  uniraum_reqstream_para(int stream, ...);

#define uniraum_cap_make(vendor,standard,version) ROAR_HOST2NET32(ROAR_STD_MAKE((vendor),(standard),(version)))
#define uniraum_cap_vendor2str(vendor) roar_stds_vendor2str((vendor))
uniraum_req_t *  uniraum_reqcaps(uniraum_cap_t * caps0, size_t len0, uniraum_cap_t * caps1, size_t len1);

int              uniraum_reqref(uniraum_req_t * req);
int              uniraum_requnref(uniraum_req_t * req);

int              uniraum_reqget_type(uniraum_req_t * req);
int              uniraum_reqget_stream(uniraum_req_t * req);
ssize_t          uniraum_reqget_datalen(uniraum_req_t * req);
void *           uniraum_reqget_data(uniraum_req_t * req);
uniraum_pos_t    uniraum_reqget_pos(uniraum_req_t * req);

uniraum_cap_t *  uniraum_reqget_caps(uniraum_req_t * req, size_t * len);

int              uniraum_reqget_stream_info(uniraum_req_t * req, int * parent, int * dir, struct roar_audio_info * info);

int              uniraum_reqget_meta(uniraum_req_t * req, const char ** key, const char ** value, int * mode);

// stream level functions:

uniraum_stream_t * uniraum_stream_new(uniraum_req_t * req);
int                uniraum_stream_ref(uniraum_stream_t * stream);
int                uniraum_stream_unref(uniraum_stream_t * stream);
#define            uniraum_stream_close(x) uniraum_stream_unref((x))
int                uniraum_stream_reqin(uniraum_stream_t * stream, uniraum_req_t * req);
uniraum_req_t *    uniraum_stream_reqout(uniraum_stream_t * stream);

int                uniraum_stream_get_stream(uniraum_stream_t * stream);
int                uniraum_stream_get_parent(uniraum_stream_t * stream);
int                uniraum_stream_get_dir(uniraum_stream_t * stream);
const struct roar_audio_info * uniraum_stream_get_auinfo(uniraum_stream_t * stream);
int                uniraum_stream_is_execed(uniraum_stream_t * stream);
int                uniraum_stream_is_eos(uniraum_stream_t * stream);

ssize_t            uniraum_stream_pending(uniraum_stream_t * stream);

// mapping level functions:

uniraum_mapping_t * uniraum_mapping_new(int stream, uniraum_dir_t dir, int type, const struct roar_audio_info * info);
int                 uniraum_mapping_ref(uniraum_mapping_t * mapping);
int                 uniraum_mapping_unref(uniraum_mapping_t * mapping);
int                 uniraum_mapping_reqin(uniraum_mapping_t * mapping, uniraum_req_t * req);
uniraum_req_t *     uniraum_mapping_reqout(uniraum_mapping_t * mapping);
int                 uniraum_mapping_datain(uniraum_mapping_t * mapping, uniraum_data_t * data);
uniraum_data_t *    uniraum_mapping_dataout(uniraum_mapping_t * mapping);

ssize_t             uniraum_mapping_pending(uniraum_mapping_t * mapping);
int                 uniraum_mapping_get_stream(uniraum_mapping_t * mapping);
uniraum_pos_t       uniraum_mapping_get_pos(uniraum_mapping_t * mapping);

// data buffer:

ssize_t             uniraum_data_get_len(uniraum_data_t * data);
void *              uniraum_data_get_data(uniraum_data_t * data);

int                 uniraum_data_get_type(uniraum_data_t * data);
#define uniraum_data_set_type(data,type) roar_buffer_set_type((data),(type))

uniraum_data_t *    uniraum_data_new(size_t len, void ** data, int type);

#define uniraum_data_ref(x) roar_buffer_ref((x))
#define uniraum_data_unref(x) roar_buffer_unref((x))

// seek points:
#define UNIRAUM_SEEKPOINT_STUB       -1
#define UNIRAUM_SEEKPOINT_EXECED     -2

uniraum_seekpoint_t * uniraum_seekpoint_new(int stream, int type);
uniraum_seekpoint_t * uniraum_seekpoint_from_req(uniraum_req_t * req, uniraum_seekpos_t pos);

int                   uniraum_seekpoint_ref(uniraum_seekpoint_t * point);
int                   uniraum_seekpoint_unref(uniraum_seekpoint_t * point);

int                   uniraum_seekpoint_set_stream(uniraum_seekpoint_t * point, int stream);
int                   uniraum_seekpoint_get_stream(uniraum_seekpoint_t * point);

int                   uniraum_seekpoint_set_type(uniraum_seekpoint_t * point, int type);
int                   uniraum_seekpoint_get_type(uniraum_seekpoint_t * point);

int            uniraum_seekpoint_set_startpos(uniraum_seekpoint_t * point, uniraum_pos_t   pos, uniraum_seekpos_t   off);
int            uniraum_seekpoint_get_startpos(uniraum_seekpoint_t * point, uniraum_pos_t * pos, uniraum_seekpos_t * off);

int            uniraum_seekpoint_set_endpos(uniraum_seekpoint_t * point, uniraum_pos_t   pos, uniraum_seekpos_t   off);
int            uniraum_seekpoint_get_endpos(uniraum_seekpoint_t * point, uniraum_pos_t * pos, uniraum_seekpos_t * off);

// seek tables:
uniraum_seektable_t * uniraum_seektable_new(int stream, size_t points);
uniraum_seektable_t * uniraum_seektable_from_req(uniraum_req_t * req, uniraum_seekpos_t pos);
uniraum_req_t       * uniraum_seektable_to_req(uniraum_seektable_t * table, uniraum_seekpos_t pos);

int                   uniraum_seektable_ref(uniraum_seektable_t * table);
int                   uniraum_seektable_unref(uniraum_seektable_t * table);

int                   uniraum_seektable_set_stream(uniraum_seektable_t * table, int stream);
int                   uniraum_seektable_get_stream(uniraum_seektable_t * table);

ssize_t               uniraum_seektable_num_points(uniraum_seektable_t * table);
uniraum_seekpoint_t * uniraum_seektable_get_point(uniraum_seektable_t * table, size_t i);
int                   uniraum_seektable_set_point(uniraum_seektable_t * table, size_t i, uniraum_seekpoint_t * point);
int                   uniraum_seektable_add_point(uniraum_seektable_t * table, uniraum_seekpoint_t * point);

#endif

//ll
