//data.c:

/*
 *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2011
 *
 *  This file is part of uniraum,
 *  a cross-platform container library, home and professional use.
 *  See README for details.
 *
 *  This file is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  uniraum is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#include "private.h"

ssize_t             uniraum_data_get_len(uniraum_data_t * data) {
 size_t len;

 if ( data == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return -1;
 }

 if ( roar_buffer_get_len(data, &len) == -1 )
  return -1;

 return len;
}

void *              uniraum_data_get_data(uniraum_data_t * data) {
 void * bufdata;

 if ( data == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 if ( roar_buffer_get_data(data, &bufdata) == -1 )
  return NULL;

 return bufdata;
}

int                 uniraum_data_get_type(uniraum_data_t * data) {
 int type;

 if ( roar_buffer_get_type(data, &type) == -1 )
  return -1;

 return type;
}

uniraum_data_t *    uniraum_data_new(size_t len, void ** data, int type) {
 uniraum_data_t * ret;
 int err;

 if ( roar_buffer_new_data(&ret, len, data) == -1 )
  return NULL;

 if ( type != -1 ) {
  if ( roar_buffer_set_type(ret, type) == -1 ) {
   err = roar_error;
   uniraum_data_unref(ret);
   roar_err_set(err);
   return NULL;
  }
 }

 return ret;
}

//ll
