//mapping.c:

/*
 *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2011
 *
 *  This file is part of uniraum,
 *  a cross-platform container library, home and professional use.
 *  See README for details.
 *
 *  This file is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  uniraum is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#include "private.h"

#define _ckmapping(m) if ( (m) == NULL ) { roar_error = ROAR_ERROR_FAULT; return -1; }

static struct mapping {
 unsigned int codec;
 int type;
 int (*init    )(uniraum_mapping_t * mapping);
 int (*deinit  )(uniraum_mapping_t * mapping);
 int (*proc_in )(uniraum_mapping_t * mapping);
 int (*proc_out)(uniraum_mapping_t * mapping);
} _g_mappings[] = {
 {ROAR_CODEC_PCM_S_LE,    ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_PCM_S_BE,    ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_PCM_S_PDP,   ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_PCM_U_LE,    ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_PCM_U_BE,    ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_PCM_U_PDP,   ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_ALAW,        ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_MULAW,       ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_AUTLAW_LE,   ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_AUTLAW_BE,   ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_MUUTLAW_LE,  ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_MUUTLAW_BE,  ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_DMX512,      ROAR_VIO_DFT_RAW,        uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_raw2req},
 {ROAR_CODEC_PCM_S_LE,    ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_S_BE,    ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_S_PDP,   ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_U_LE,    ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_U_BE,    ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_U_PDP,   ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_ALAW,        ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_MULAW,       ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_AUTLAW_LE,   ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_AUTLAW_BE,   ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_MUUTLAW_LE,  ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_MUUTLAW_BE,  ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_DMX512,      ROAR_VIO_DFT_PACKET,     uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_S_LE,    ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_S_BE,    ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_S_PDP,   ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_U_LE,    ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_U_BE,    ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_PCM_U_PDP,   ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_ALAW,        ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_MULAW,       ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_AUTLAW_LE,   ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_AUTLAW_BE,   ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_MUUTLAW_LE,  ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_MUUTLAW_BE,  ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},
 {ROAR_CODEC_DMX512,      ROAR_VIO_DFT_UNFRAMED,   uniraum_mapper_pcm_init, uniraum_mapper_pcm_deinit,
                                                   uniraum_mapper_pcm_req2packet, uniraum_mapper_pcm_packet2req},

#ifdef HAVE_LIBOGG
 {ROAR_CODEC_RAUM_VORBIS, ROAR_VIO_DFT_OGG_PACKET, uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2packet, uniraum_mapper_xiph_packet2req},
 {ROAR_CODEC_RAUM_VORBIS, ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2page, uniraum_mapper_xiph_page2req},
 {ROAR_CODEC_RAUM_VORBIS, ROAR_VIO_DFT_RAW,        uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2raw, uniraum_mapper_xiph_raw2req},
 {ROAR_CODEC_RAUM_VORBIS, ROAR_VIO_DFT_PACKET,     uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2raw, uniraum_mapper_xiph_raw2req},
 {ROAR_CODEC_RAUM_FLAC,   ROAR_VIO_DFT_OGG_PACKET, uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2packet, uniraum_mapper_xiph_packet2req},
 {ROAR_CODEC_RAUM_FLAC,   ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2page, uniraum_mapper_xiph_page2req},
 {ROAR_CODEC_RAUM_FLAC,   ROAR_VIO_DFT_RAW,        uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2raw, uniraum_mapper_xiph_raw2req},
 {ROAR_CODEC_RAUM_FLAC,   ROAR_VIO_DFT_PACKET,     uniraum_mapper_xiph_init, uniraum_mapper_xiph_deinit,
                                                   uniraum_mapper_xiph_req2raw, uniraum_mapper_xiph_raw2req},

 {ROAR_CODEC_OGG,         ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG_VORBIS,  ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG_SPEEX,   ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG_FLAC,    ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG_GENERAL, ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG_CELT,    ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG_OPUS,    ROAR_VIO_DFT_OGG_PAGE,   uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2page, uniraum_mapper_ogg_page2req},
 {ROAR_CODEC_OGG,         ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
 {ROAR_CODEC_OGG_VORBIS,  ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
 {ROAR_CODEC_OGG_SPEEX,   ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
 {ROAR_CODEC_OGG_FLAC,    ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
 {ROAR_CODEC_OGG_GENERAL, ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
 {ROAR_CODEC_OGG_CELT,    ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
 {ROAR_CODEC_OGG_OPUS,    ROAR_VIO_DFT_RAW,        uniraum_mapper_ogg_init, uniraum_mapper_ogg_deinit,
                                                   uniraum_mapper_ogg_req2raw, uniraum_mapper_ogg_raw2req},
#endif
};

static const struct mapping * _get_mapping(int type, const struct roar_audio_info * info) {
 size_t i;

 if ( info == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 for (i = 0; i < (sizeof(_g_mappings)/sizeof(*_g_mappings)); i++)
  if ( _g_mappings[i].codec == info->codec && _g_mappings[i].type == type )
   return &(_g_mappings[i]);

 ROAR_DBG("_get_mapping(*) = NULL // error=NOTSUP");
 roar_err_set(ROAR_ERROR_NOTSUP);
 return NULL;
}

static int _ck_type(uniraum_dir_t dir, int type, const struct roar_audio_info * info) {
 const struct mapping * mapping;

 if ( info == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return -1;
 }

 mapping = _get_mapping(type, info);

 if ( mapping == NULL ) {
  ROAR_DBG("_ck_type(*) = 0 // error=NOTSUP");
  roar_err_set(ROAR_ERROR_NOTSUP);
  return 0;
 }

 if ( dir == UNIRAUM_IN ) {
  if ( mapping->proc_in != NULL )
   return 1;
 } else if ( dir == UNIRAUM_OUT ) {
  if ( mapping->proc_out != NULL )
   return 1;
 } else {
  ROAR_DBG("_ck_type(*) = -1 // error=NOTSUP");
  roar_err_set(ROAR_ERROR_NOTSUP);
  return -1;
 }

 ROAR_DBG("_ck_type(*) = 0 // error=NOTSUP");
 roar_err_set(ROAR_ERROR_NOTSUP);
 return 0;
}

uniraum_mapping_t * uniraum_mapping_new(int stream, uniraum_dir_t dir, int type, const struct roar_audio_info * info) {
 uniraum_mapping_t * ret = NULL;
 const struct mapping * mapper;

 if ( info == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 if ( stream == -1 ) {
  roar_err_set(ROAR_ERROR_INVAL);
  return NULL;
 }

 // this will also check dir argument.
 if ( _ck_type(dir, type, info) != 1 )
  return NULL;

 mapper = _get_mapping(type, info);
 if ( mapper == NULL )
  return NULL;

 ret = roar_mm_malloc(sizeof(uniraum_mapping_t));
 if (ret == NULL)
  return NULL;

 memset(ret, 0, sizeof(uniraum_mapping_t));

 ret->refc   = 1;
 ret->stream = stream;
 ret->dir    = dir;
 ret->type   = type;

 ret->req_queue_fill  = 0;
 ret->data_queue_fill = 0;

 memcpy(&(ret->info), info, sizeof(ret->info));

 if ( mapper->init != NULL ) {
  if ( mapper->init(ret) != 0 ) {
   roar_mm_free(ret);
  }
 }

 return ret;
}

int                 uniraum_mapping_ref(uniraum_mapping_t * mapping) {
 _ckmapping(mapping);

 mapping->refc++;

 return 0;
}

int                 uniraum_mapping_unref(uniraum_mapping_t * mapping) {
 const struct mapping * mapper;
 size_t i;

 _ckmapping(mapping);

 mapping->refc--;

 if ( mapping->refc == 0 ) {
  mapper = _get_mapping(mapping->type, &(mapping->info));
  if ( mapper == NULL ) {
   ROAR_ERR("uniraum_mapping_unref(mapping=%p): Can not find a valid mapper. Must be a memory corruption.", mapping);
   roar_panic(ROAR_FATAL_ERROR_MEMORY_CORRUPTION, NULL);
  } else {
   if ( mapper->deinit != NULL ) {
    mapper->deinit(mapping);
   }
  }

  for (i = 0; i < mapping->req_queue_fill; i++)
   uniraum_requnref(mapping->req_queue[i]);
  for (i = 0; i < mapping->data_queue_fill; i++)
   uniraum_data_unref(mapping->data_queue[i]);

  if ( mapping->state.buf != NULL )
   uniraum_data_unref(mapping->state.buf);

  roar_mm_free(mapping);
 }

 return 0;
}

static int _mapping_proc(uniraum_mapping_t * mapping) {
 const struct mapping * mapper;

 ROAR_DBG("_mapping_proc(mapping=%p) = ?", mapping);

 _ckmapping(mapping);

 mapper = _get_mapping(mapping->type, &(mapping->info));
 if ( mapper == NULL )
  return -1;

 ROAR_DBG("_mapping_proc(mapping=%p): mapper=%p", mapping, mapper);

 switch (mapping->dir) {
  case UNIRAUM_IN:
    if ( mapping->req_queue_fill == 0 ) {
     ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=NODATA (no data in request queue)", mapping);
     roar_err_set(ROAR_ERROR_NODATA);
     return -1;
    }

    if ( mapping->data_queue_fill == _REQ_QUEUE_LEN ) {
     ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=AGAIN (data queue full)", mapping);
     roar_err_set(ROAR_ERROR_AGAIN);
     return -1;
    }

    if ( mapper->proc_in == NULL ) {
     ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=NOTSUP", mapping);
     roar_err_set(ROAR_ERROR_NOTSUP);
     return -1;
    }

    return mapper->proc_in(mapping);
   break;
  case UNIRAUM_OUT:
    if ( mapping->data_queue_fill == 0 ) {
     ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=NODATA (no data in data queue)", mapping);
     roar_err_set(ROAR_ERROR_NODATA);
     return -1;
    }

    if ( mapping->req_queue_fill == _DATA_QUEUE_LEN ) {
     ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=AGAIN (request queue full)", mapping);
     roar_err_set(ROAR_ERROR_AGAIN);
     return -1;
    }

    if ( mapper->proc_out == NULL ) {
     ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=NOTSUP", mapping);
     roar_err_set(ROAR_ERROR_NOTSUP);
     return -1;
    }

    return mapper->proc_out(mapping);
   break;
  default:
    ROAR_DBG("_mapping_proc(mapping=%p) = -1 // error=NOSYS", mapping);
    roar_err_set(ROAR_ERROR_NOSYS);
    return -1;
   break;
 }
}

int                 uniraum_mapping_reqin(uniraum_mapping_t * mapping, uniraum_req_t * req) {
 int ret;

 _ckmapping(mapping);

 if ( mapping->dir != UNIRAUM_IN ) {
  roar_err_set(ROAR_ERROR_INVAL);
  return -1;
 }

 ret = uniraum_mapping_pushreq(mapping, req);

 if ( ret == 0 )
  _mapping_proc(mapping);

 return ret;
}

int                 uniraum_mapping_pushreq(uniraum_mapping_t * mapping, uniraum_req_t * req) {
 int stream;

 _ckmapping(mapping);

 if ( mapping->req_queue_fill == _REQ_QUEUE_LEN ) {
  _mapping_proc(mapping);
  if ( mapping->req_queue_fill == _REQ_QUEUE_LEN ) {
   roar_err_set(ROAR_ERROR_AGAIN);
   return -1;
  }
 }

 stream = uniraum_reqget_stream(req);
 if ( stream == -1 )
  return -1;

 if ( stream != mapping->stream ) {
  roar_err_set(ROAR_ERROR_INVAL);
  return -1;
 }

 if ( uniraum_reqref(req) == -1 )
  return -1;

 mapping->req_queue[mapping->req_queue_fill++] = req;

 return 0;
}

uniraum_req_t *     uniraum_mapping_reqout(uniraum_mapping_t * mapping) {
 if ( mapping == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 if ( mapping->req_queue_fill == 0 ) {
  _mapping_proc(mapping);
 }

 return uniraum_mapping_shiftreq(mapping);
}

uniraum_req_t *     uniraum_mapping_shiftreq(uniraum_mapping_t * mapping) {
 uniraum_req_t * ret;
 size_t i;

 if ( mapping == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 if ( mapping->req_queue_fill == 0 ) {
  roar_err_set(ROAR_ERROR_AGAIN);
  return NULL;
 }

 ret = mapping->req_queue[0];

 mapping->req_queue_fill--;

 for (i = 0; i < mapping->req_queue_fill; i++)
  mapping->req_queue[i] = mapping->req_queue[i+1];

 return ret;
}

int                 uniraum_mapping_datain(uniraum_mapping_t * mapping, uniraum_data_t * data) {
 int ret;

 _ckmapping(mapping);

 if ( mapping->dir != UNIRAUM_OUT ) {
  roar_err_set(ROAR_ERROR_INVAL);
  return -1;
 }

 ret = uniraum_mapping_pushdata(mapping, data);

 if ( ret == 0 )
  _mapping_proc(mapping);

 return ret;
}

int                 uniraum_mapping_pushdata(uniraum_mapping_t * mapping, uniraum_data_t * data) {
 int type;

 _ckmapping(mapping);

 if ( mapping->data_queue_fill == _DATA_QUEUE_LEN ) {
  _mapping_proc(mapping);
  if ( mapping->data_queue_fill == _DATA_QUEUE_LEN ) {
   roar_err_set(ROAR_ERROR_AGAIN);
   return -1;
  }
 }

 if ( roar_buffer_get_type(data, &type) == -1 )
  return -1;

 if ( _ck_type(mapping->dir, type, &(mapping->info)) != 1 )
  return -1;

 if ( roar_buffer_ref(data) == -1 )
  return -1;

 mapping->data_queue[mapping->data_queue_fill++] = data;

 return 0;
}

uniraum_data_t *    uniraum_mapping_dataout(uniraum_mapping_t * mapping) {
 if ( mapping == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 if ( mapping->data_queue_fill == 0 ) {
  _mapping_proc(mapping);
 }

 return uniraum_mapping_shiftdata(mapping);
}

uniraum_data_t *    uniraum_mapping_shiftdata(uniraum_mapping_t * mapping) {
 uniraum_data_t * ret;
 size_t i;

 if ( mapping == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 if ( mapping->data_queue_fill == 0 ) {
  roar_err_set(ROAR_ERROR_AGAIN);
  return NULL;
 }

 ret = mapping->data_queue[0];

 mapping->data_queue_fill--;

 for (i = 0; i < mapping->data_queue_fill; i++)
  mapping->data_queue[i] = mapping->data_queue[i+1];

 return ret;
}

ssize_t             uniraum_mapping_pending(uniraum_mapping_t * mapping) {
 _ckmapping(mapping);

 _mapping_proc(mapping);

 if ( mapping->dir == UNIRAUM_IN ) {
  return mapping->data_queue_fill;
 } else {
  return mapping->req_queue_fill;
 }
}

int                 uniraum_mapping_get_stream(uniraum_mapping_t * mapping) {
 _ckmapping(mapping);

 return mapping->stream;
}

uniraum_pos_t       uniraum_mapping_get_pos(uniraum_mapping_t * mapping) {
 if ( mapping == NULL ) {
  roar_error = ROAR_ERROR_FAULT;
  return 0;
 }

 return mapping->state.pos;
}

//ll
