//private.c:

/*
 *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2011
 *
 *  This file is part of uniraum,
 *  a cross-platform container library, home and professional use.
 *  See README for details.
 *
 *  This file is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  uniraum is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#ifndef _UNIRAUM_PRIVATE_H_
#define _UNIRAUM_PRIVATE_H_

#include "../include/uniraum/uniraum.h"
#include "../config.h"

#define _RAUM_MAGIC     "RAUM-CF0"
#define _RAUM_MAGIC_LEN 8

#define _QUEUE_LEN      8
#define _REQ_QUEUE_LEN  _QUEUE_LEN
#define _DATA_QUEUE_LEN _QUEUE_LEN

#define UNIRAUM_STFLAG_NONE      0x00
#define UNIRAUM_STFLAG_CMD       0x01 /* Commands possition/Other Event */
#define UNIRAUM_STFLAG_END       0x02 /* Event has end time */
#define UNIRAUM_STFLAG_STREAMID  0x04 /* point has a stream ID */
#define UNIRAUM_STFLAG_F3        0x08
#define UNIRAUM_STFLAG_F4        0x10
#define UNIRAUM_STFLAG_F5        0x20

#define UNIRAUM_STFLAG_SIZEMASK  0xC0
#define UNIRAUM_STFLAG_SIZE16INC 0x00
#define UNIRAUM_STFLAG_SIZE32    0x40
#define UNIRAUM_STFLAG_SIZE64    0x80
#define UNIRAUM_STFLAG_SIZE32INC 0xC0

struct uniraum_file {
 size_t refc;
 int error;
 int o_flags;
 struct roar_vio_calls   vio_store;
 struct roar_vio_calls   vio_re;
 struct roar_vio_calls * vio;
 int execed;

 uniraum_req_t * req_queue_out[_REQ_QUEUE_LEN];
 size_t          req_queue_out_fill;
 uniraum_req_t * req_queue_in[_REQ_QUEUE_LEN];
 size_t          req_queue_in_fill;
};

struct uniraum_req {
 size_t refc;
 struct roar_message mes;
 void * long_data;
 void * buf;
};

struct uniraum_stream {
 size_t refc;
 int flags;
 int stream;
 int parent;
 int dir;
 struct roar_audio_info info;
 uniraum_req_t * req_queue[_REQ_QUEUE_LEN];
 size_t          req_queue_fill;
};

struct uniraum_mapping {
 size_t refc;
 uniraum_dir_t dir;
 int type;
 int stream;
 struct roar_audio_info info;
 uniraum_req_t * req_queue[_REQ_QUEUE_LEN];
 size_t          req_queue_fill;
 uniraum_data_t* data_queue[_DATA_QUEUE_LEN];
 size_t          data_queue_fill;

 // some internal state infos to be used by the mappings:
 struct {
  size_t           bpf;
  size_t           iolen;
  uniraum_pos_t    pos;
  uniraum_data_t * buf;
  void           * vp;
 } state;
};

struct uniraum_seektable {
 size_t refc;
 int                    version;
 int                    stream;
 size_t                 points;
 uniraum_seekpoint_t ** point;
 size_t                 req_size; // this is to store how much /padding/ we need to add
                                  // value of zero indecates that there is no size forced
                                  // padding is how much data is left according to
                                  // the message after all points are read/written
 size_t                 padding;  // the padding added on last write/read
 int                    use_padding;
};

struct uniraum_seekpoint {
 size_t refc;
 int stream;
 int type;
 unsigned int flags;
 uniraum_pos_t pos_start;
 uniraum_pos_t pos_end;
 uniraum_seekpos_t offset_start;
 uniraum_seekpos_t offset_end;
};

// req stuff:
uniraum_req_t *  uniraum_reqnewempty(void);

// stuff for mapping:
int                 uniraum_mapping_pushreq(uniraum_mapping_t * mapping, uniraum_req_t * req);
uniraum_req_t *     uniraum_mapping_shiftreq(uniraum_mapping_t * mapping);
int                 uniraum_mapping_pushdata(uniraum_mapping_t * mapping, uniraum_data_t * data);
uniraum_data_t *    uniraum_mapping_shiftdata(uniraum_mapping_t * mapping);

// mappers:
int uniraum_mapper_pcm_init(uniraum_mapping_t * mapping);
int uniraum_mapper_pcm_deinit(uniraum_mapping_t * mapping);
int uniraum_mapper_pcm_packet2req(uniraum_mapping_t * mapping);
int uniraum_mapper_pcm_raw2req(uniraum_mapping_t * mapping);
int uniraum_mapper_pcm_req2packet(uniraum_mapping_t * mapping);

int uniraum_mapper_xiph_init(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_deinit(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_packet2req(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_page2req(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_raw2req(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_req2packet(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_req2page(uniraum_mapping_t * mapping);
int uniraum_mapper_xiph_req2raw(uniraum_mapping_t * mapping);

int uniraum_mapper_ogg_init(uniraum_mapping_t * mapping);
int uniraum_mapper_ogg_deinit(uniraum_mapping_t * mapping);
int uniraum_mapper_ogg_page2req(uniraum_mapping_t * mapping);
int uniraum_mapper_ogg_raw2req(uniraum_mapping_t * mapping);
int uniraum_mapper_ogg_req2page(uniraum_mapping_t * mapping);
int uniraum_mapper_ogg_req2raw(uniraum_mapping_t * mapping);

#endif

//ll
