//seekpoint.c:

/*
 *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2011
 *
 *  This file is part of uniraum,
 *  a cross-platform container library, home and professional use.
 *  See README for details.
 *
 *  This file is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  uniraum is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#include "private.h"

#define _ckpoint()  if ( point == NULL ) { roar_err_set(ROAR_ERROR_FAULT); return -1; }

uniraum_seekpoint_t * uniraum_seekpoint_new(int stream, int type) {
 uniraum_seekpoint_t * ret = roar_mm_malloc(sizeof(uniraum_seekpoint_t));

 if ( ret == NULL )
  return NULL;

 memset(ret, 0, sizeof(uniraum_seekpoint_t));

 ret->refc   = 1;
 ret->stream = stream;
 ret->type   = type;

 return ret;
}

uniraum_seekpoint_t * uniraum_seekpoint_from_req(uniraum_req_t * req, uniraum_seekpos_t pos) {
 uniraum_seekpoint_t * ret;
 int reqtype, reqstream, reqpos;

 if ( req == NULL ) {
  roar_err_set(ROAR_ERROR_FAULT);
  return NULL;
 }

 reqtype   = uniraum_reqget_type(req);
 reqstream = uniraum_reqget_stream(req);
 reqpos    = uniraum_reqget_pos(req);

 ret = uniraum_seekpoint_new(reqstream, reqtype);
 if ( ret == NULL )
  return NULL;

 ret->pos_start    = reqpos;
 ret->offset_start = pos;

 return ret;
}

int                uniraum_seekpoint_ref(uniraum_seekpoint_t * point) {
 _ckpoint();

 point->refc++;

 return 0;
}

int                uniraum_seekpoint_unref(uniraum_seekpoint_t * point) {
 _ckpoint();

 point->refc--;

 if ( point->refc == 0 ) {
  roar_mm_free(point);
 }

 return 0;
}

int                   uniraum_seekpoint_set_stream(uniraum_seekpoint_t * point, int stream) {
 _ckpoint();

 point->stream = stream;
 return 0;
}

int                   uniraum_seekpoint_get_stream(uniraum_seekpoint_t * point) {
 _ckpoint();

 return point->stream;
}

int                   uniraum_seekpoint_set_type(uniraum_seekpoint_t * point, int type) {
 _ckpoint();

 point->type = type;
 return 0;
}

int                   uniraum_seekpoint_get_type(uniraum_seekpoint_t * point) {
 _ckpoint();

 return point->type;
}

int            uniraum_seekpoint_set_startpos(uniraum_seekpoint_t * point, uniraum_pos_t   pos, uniraum_seekpos_t   off) {
 _ckpoint();

 point->pos_start    = pos;
 point->offset_start = off;

 return 0;
}

int            uniraum_seekpoint_get_startpos(uniraum_seekpoint_t * point, uniraum_pos_t * pos, uniraum_seekpos_t * off) {
 _ckpoint();

 if ( pos != NULL )
  *pos = point->pos_start;

 if ( off != NULL )
  *off = point->offset_start;

 return 0;
}

int                   uniraum_seekpoint_set_endpos(uniraum_seekpoint_t * point, uniraum_pos_t pos, uniraum_seekpos_t off) {
 _ckpoint();

 point->pos_end    = pos;
 point->offset_end = off;

 point->flags     |= UNIRAUM_STFLAG_END;

 return 0;
}

int            uniraum_seekpoint_get_endpos(uniraum_seekpoint_t * point, uniraum_pos_t * pos, uniraum_seekpos_t * off) {
 _ckpoint();

 if ( !(point->flags & UNIRAUM_STFLAG_END) ) {
  roar_err_set(ROAR_ERROR_NODATA);
  return -1;
 }

 if ( pos != NULL )
  *pos = point->pos_end;

 if ( off != NULL )
  *off = point->offset_end;

 return 0;
}

//ll
